#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# VDaT approximating BTM - Coherence heuristic. Figure 8 in pisha2020approximate.

import math
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider, CheckButtons, RadioButtons

from common import *
from btm import *
from vdat import *
from examples3d import halfplane_btm

fig, ax = plt.subplots(figsize=[5, 3])
    
btmapx_nocoh = VDaT_BTM_Apx(coherence=False)
btmapx_coh = VDaT_BTM_Apx(coherence=True)

ringdefs = defaultringdefs(9, 64, 44100, 345.0)
params = {'fs': 384000, 'c': 345.0, 'irlen': 4096, 'totalirlen': 500000, 'goodnumint': False}

setup_ax(ax)
ax.set_ylim([-15.0, 0.0])
zi = 0.5
zd = -0.5
x = -0.02

verts, tris, edges = halfplane_btm(x, 0.0)
mags, freqs, _, _ = call_btm(verts, tris, edges, [0,0,zi], [0,0,zd], True, params)
mags = todb(mags)
ax.plot(freqs, mags, color='blue', marker='', label='BTM')

rxm = vdat_2d_rxm(x, ringdefs)
ztotal = abs(zd - zi)

vmags_nocoh = todb(btmapx_nocoh.rxm_to_resp(freqs, ringdefs, rxm, ztotal))
ax.plot(freqs, vmags_nocoh, color='green', marker='', linestyle='-.', label='VDaT, no coherence')

vmags_coh = todb(btmapx_coh.rxm_to_resp(freqs, ringdefs, rxm, ztotal))
ax.plot(freqs, vmags_coh, color='orange', marker='', linestyle='--', label='VDaT, with coherence')
    
ax.legend(loc='lower left')
ax.set_title('x = {:.2f}, zi = {:.2f}, zd = {:.2f}'.format(abs(x), zi, zd))

plt.tight_layout()
plt.show()
