#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# VDaT approximating BTM - Random disks, manually controlled
# Use the on-screen controls to generate and view disks.
# This script was used to partially generate Figure 5 in pisha2020approximate.

import random

import math
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Button, Slider, CheckButtons, RadioButtons

from common import *
from btm import *
from btm_montecarlo import *

plt.figure(figsize=(10, 6), dpi=100)
simax = plt.axes((0.05, 0.2, 0.4, 0.7))
magax = plt.axes((0.55, 0.1, 0.4, 0.8))

howmanyismany = 20

btnAnother = Button(plt.axes([0.05, 0.07, 0.07, 0.07]), 'Another')
btnMany = Button(plt.axes([0.12, 0.07, 0.07, 0.07]), str(howmanyismany) + ' more')
btnReset = Button(plt.axes([0.19, 0.07, 0.07, 0.07]), 'Reset')
btnSave = Button(plt.axes([0.26, 0.07, 0.07, 0.07]), 'Save')
sldZoom = Slider(plt.axes([0.05, 0.04, 0.4, 0.03], facecolor='purple'), 
    'Zoom', -2.0, 2.0, valinit=-0.3, valstep=0.01)

bm = None
def sldZoomChanged(_):
    global bm
    zoom = pow(10.0, -sldZoom.val)
    rout = 0.4 * zoom
    bm = BTMMonteCarlo(1.0, -1.0, rout*0.5, rout, 'manual', 'orange')
sldZoomChanged(None)
sldZoom.on_changed(sldZoomChanged)

def redraw():
    verts, tris, edges = bm.get_lastobject()
    
    thetas = np.arange(0, math.tau, 0.01)
    xs = np.cos(thetas)
    ys = np.sin(thetas)
    simax.clear()
    zoom = pow(10.0, -sldZoom.val)
    simax.set_xlim(-zoom, zoom)
    simax.set_ylim(-zoom, zoom)
    rin, rout = bm.get_rs()
    simax.plot(rin * xs, rin * ys, marker='', linestyle='--', color='green')
    simax.plot(rout * xs, rout * ys, marker='', linestyle='--', color='red')
    for e in edges:
        seg = [verts[e['i1']], verts[e['i2']]]
        simax.plot([v[0] for v in seg], [v[1] for v in seg], marker='o', color='darkgoldenrod')

    magax.clear()
    magax.set_xscale('log')
    magax.set_xlim(20, 25000)
    magax.set_ylim(-60, 40)
    
    #magax.plot([bm.get_knee()] * 2, [-100, 100], marker='', color='lightgrey')
    
    magax.plot(bm.get_freqs(), bm.get_lastres(), marker='', linestyle='--', 
        color='darkgoldenrod', label='Single disk')   
    magax.plot(bm.get_freqs(), bm.get_avg(), marker='', color='blue', label='Average of 300')    
    magax.set_xlabel('Frequency (Hz)')
    magax.set_ylabel('Amplitude response (dB)')
    magax.set_title('BTM amplitude response for disk')
    magax.legend(loc='lower left')
    plt.gcf().canvas.draw_idle()

btnAnother.on_clicked(lambda _: bm.another() or redraw())
btnMany.on_clicked(lambda _: bm.another(howmanyismany) or redraw())
btnReset.on_clicked(lambda _: bm.another(reset=True) or redraw())
btnSave.on_clicked(lambda _: bm.save())

redraw()
plt.show()
