#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# Results - Non-shadowed diffraction by a half-plane. Figure 12 in pisha2020approximate.

import math
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider, CheckButtons, RadioButtons

from common import *
from btm import *
from vdat import *
from examples3d import halfplane_btm

fig, ax = plt.subplots(figsize=[5, 3.3])
    
btmapx = VDaT_BTM_Apx()

ringdefs = defaultringdefs(9, 64, 44100, 345.0)
params = {'fs': 384000, 'c': 345.0, 'irlen': 4096, 'totalirlen': 500000, 'goodnumint': False}

setup_ax(ax)
ax.set_ylim([-15.0, 0.0])
zi = 1.0
zd = -1.0

fwlsd_accum = 0.0
fwlsd_counter = 0
dolabel = True
for x, lblx, va in zip([0.02, 0.1, 1.0], [4000, 2000, 30], ['top', 'top', 'bottom']):
    verts, tris, edges = halfplane_btm(x, 0.0)
    mags, freqs, _, _ = call_btm(verts, tris, edges, [0,0,zi], [0,0,zd], True, params)
    mags = todb(mags)
    ax.plot(freqs, mags, color='blue', marker='', label='BTM' if dolabel else None)

    rxm = vdat_2d_rxm(x, ringdefs)
    ztotal = abs(zd - zi)
    vmags = todb(btmapx.rxm_to_resp(freqs, ringdefs, rxm, ztotal))
    ax.plot(freqs, vmags, color='orange', marker='', linestyle='--', label='VDaT' if dolabel else None)
    ax.text(lblx, mags[floor(lblx / freqs[1])], 'x = ' + str(x), fontsize=12.0, va=va)
    
    fwlsd = FW_LSD(freqs, mags, vmags, True)
    fwlsd_accum += fwlsd
    fwlsd_counter += 1
    print('zi {} zd {} x {}: fwlsd {:.2f} dB'.format(zi, zd, x, fwlsd))
    
    dolabel = False
ax.legend(loc='lower left')
ax.set_title('zi = {:.2f}, zd = {:.2f}'.format(zi, zd))

print('Average FW-LSD: {:.2f} dB'.format(fwlsd_accum / float(fwlsd_counter)))

plt.tight_layout()
plt.show()
