#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# Results - Shadowed diffraction by a half-plane. Figure 11 in pisha2020approximate.

import math
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider, CheckButtons, RadioButtons

from common import *
from btm import *
from vdat import *
from examples3d import halfplane_btm

fig, axs = plt.subplots(3, 2, figsize=[6, 8])
axs = axs.transpose().reshape(-1)

pltparams = [
    {'zi': -5.0, 'zd': 5.0, 'ymax': -15.0},
    {'zi': -0.5, 'zd': 0.5, 'ymax': 5.0},
    {'zi': -0.05, 'zd': 0.05, 'ymax': 20.0},
    {'zi': -0.1, 'zd': 9.9, 'ymax': -15.0},
    {'zi': -0.05, 'zd': 0.95, 'ymax': 5.0},
    {'zi': -0.01, 'zd': 0.09, 'ymax': 20.0}]
    
    
btmapx = VDaT_BTM_Apx()

ringdefs = defaultringdefs(9, 64, 44100, 345.0)
params = {'fs': 384000, 'c': 345.0, 'irlen': 4096, 'totalirlen': 500000, 'goodnumint': False}

fwlsd_accum = 0.0
fwlsd_counter = 0
fwlsd_max = 0.0
for a, ax in enumerate(axs):
    setup_ax(ax)
    ax.set_ylim([-60.0, pltparams[a]['ymax']])
    zi = pltparams[a]['zi']
    zd = pltparams[a]['zd']
    
    dolabel = True
    for x in [-2.0, -1.0, -0.5, -0.2, -0.1, -0.05, -0.02]:
        verts, tris, edges = halfplane_btm(x, 0.0)
        mags, freqs, _, _ = call_btm(verts, tris, edges, [0,0,zi], [0,0,zd], True, params)
        mags = todb(mags)
        ax.plot(freqs, mags, color='blue', marker='', label='BTM' if dolabel else None)

        rxm = vdat_2d_rxm(x, ringdefs)
        ztotal = abs(zd - zi)
        vmags = todb(btmapx.rxm_to_resp(freqs, ringdefs, rxm, ztotal))
        ax.plot(freqs, vmags, color='orange', marker='', linestyle='--', label='VDaT' if dolabel else None)
        
        fwlsd = FW_LSD(freqs, mags, vmags, True)
        fwlsd_accum += fwlsd
        fwlsd_counter += 1
        fwlsd_max = max(fwlsd_max, fwlsd)
        print('zi {} zd {} x {}: FW-LSD {:.2f} dB'.format(zi, zd, x, fwlsd))
        
        dolabel = False
    ax.legend(loc='lower left')
    ax.set_title('zi = {:.2f}, zd = {:.2f}'.format(zi, zd))

print('Average FW-LSD: {:.2f} dB / Max FW-LSD: {:.2f} dB'.format(
    fwlsd_accum / float(fwlsd_counter), fwlsd_max))

plt.tight_layout()
plt.show()
