#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# VDaT approximating BTM - Generation of data for parameter optimization.
# Takes some time to run.

import random
import math
import numpy as np

from common import *
from btm import *
from btm_montecarlo import *

#High quality params
fs, irlen, totalirlen, nanother = 384000, 2048, 65536, 500
#Low quality params
#fs, irlen, totalirlen, nanother = 192000, 1024, 65536, 100
#Testing, very low quality
#fs, irlen, totalirlen, nanother = 44100, 512, 65536, 5

rout = 16.384
seed = 1234567
colors = []
while rout > 0.02:
    print('Radius ' + str(rout))
    color = rand_color(colors)
    colors.append(color)
    def run_bmc(zi, zd, outside):
        if outside and abs(zi - zd) >= 10.0 and rout < 0.05:
            print('Skipping 20p0_outside_sym_0p032.pickle.no')
            return
        label = str(abs(zi - zd)).replace('.', 'p').replace('-', 'm')
        label += '_' + ('outside' if outside else 'inside')
        label += '_' + ('sym' if zi == -zd else 'asym')
        print(label)
        bm = BTMMonteCarlo(zi, zd, None, rout, label, color, seed, outside,
            fs=fs, irlen=irlen, totalirlen=totalirlen)
        bm.another(nanother)
        bm.save()
    for mult in [0.1, 1.0, 10.0]:
        run_bmc(1.0*mult, -1.0*mult, False)
        run_bmc(0.01, -2.0*mult + 0.01, False)
        run_bmc(1.0*mult, -1.0*mult, True)
    rout *= 0.5
    seed += 1
    
print('Done')
