#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# VDaT approximating BTM - Parameter optimizer.
# To see it converge, change some of these initial parameters below before
# starting, otherwise not much will happen.
# optimizer_gen_data.py must have been run first to generate the data used here.

initparams = [1.44299654, 0.91902257, -0.32802643, 1.30756273, -0.33781434, 1.58287113,
    0.77353521, 1.04458582, 0.79911644, -1.0509826, 0.68170791, 0.34534812]

# Which parameter indexes are being optimized.
p_start = 0
p_end = 12


import math, random
import numpy as np
import sys, pickle

from common import *
from vdat import VDaT_BTM_Apx

data = []
freqs, where_freqs_in_range, l1wt = None, None, None

print('Loading data...')
for f in sys.argv[1:]:
    with open(f, 'rb') as p:
        d = pickle.load(p)
        f = np.array(d['freqs'])
        if freqs is None:
            freqs = audible_only(f)
            l1wt = 1000.0 / freqs
        d['mags'] = audible_only(f, d['mags'])
        data.append(d)

if len(data) == 0:
    print('This script should be run like "python3 optimizer_run.py out/*.pickle"')
    sys.exit(1)

def eval_perf(params):
    v = VDaT_BTM_Apx(*list(params))
    try:
        totalerr = 0.0
        for d in data:
            ztotal = abs(d['zi']) + abs(d['zd'])
            rout = d['rout']
            
            mags_vdat = v.dring_outside(freqs, ztotal, rout) if d['outside'] \
                else v.dring_inside(freqs, ztotal, rout)
            mags_vdat = todb(mags_vdat)
            err = np.sum(np.multiply(l1wt, np.absolute(d['mags'] - mags_vdat)))
            totalerr += err
        return totalerr
    except (ValueError, TypeError) as e:
        return 1000000000000.0

def line_search(bestparams, besterr, deltaparams, label):
    dp = 0.1
    initerr = besterr
    while dp >= 0.0001:
        gotbetter = False
        for sg in (1.0, -1.0):
            err = eval_perf(bestparams + sg * dp * deltaparams)
            if err < besterr:
                bestparams += sg * dp * deltaparams
                besterr = err
                gotbetter = True
                break
        dp *= 1.5 if gotbetter else 0.3
    if besterr < initerr:
        print('Improved from {:.5f} to {:.5f} with {}'.format(initerr, besterr, label))
    else:
        print('Unable to improve error {:.5f} with {}'.format(initerr, label))
    return bestparams, besterr

bestparams = np.array(initparams)
besterr = eval_perf(bestparams)
lastp = -1
p = lastp

while True:
    print('Best parameters currently:', bestparams)
    # Single parameter line search
    while p == lastp:
        p = random.randrange(p_start, p_end)
    lastp = p
    deltaparams = np.zeros_like(bestparams)
    deltaparams[p] = 1.0
    bestparams, besterr = line_search(bestparams, besterr, deltaparams, '1-p search')
    # Random line search
    deltaparams = np.zeros_like(bestparams)
    for p in range(p_start, p_end):
        deltaparams[p] = pow(2.0, 5.0 * random.random() - 2.0) * (random.random() - 0.5)
    bestparams, besterr = line_search(bestparams, besterr, deltaparams, 'random search')
    
        
